function [TtB] = timetoboundary(CoPML, CoPAP, Fsamp, Bound)
%   [TtB] = timetoboundary(CoPML, CoPAP, Fsamp, Bound);
%   Routine to extract the time-to-boundary (TtB)
%   (aka Virtual Time-to-Contact, VTC)
%   function according to Slobounov et al. 1997
%
%   INPUT:
%   CoPML = CoP ML coordinate {m} [numsamples x 1];
%   CoPAP = CoM AP coordinate {m} [numsamples x 1];
%   Fsamp = Sampling rate {samples/s}[1 x 1] (default = 100 samples/s);
%   Bound = Boundary Limits {m} [2 x 1] (default = 2 x range of each coordinate);
%
%   OUTPUT:
%   TtB = time to boundary function {s} [numsamples x 1];
%
%   DEPENDENCIES:
%   roots function, in the polynomial toolbox
%   
%   REFERENCES:
%   S.M. Slobounov, E.S. Slobounova and K.M. Newell, Virtual
%   time-to-collision and human postural control, J Mot Behav 29 (1997),
%   pp. 263281
%
%   This routine was written by:
%   Maurizio Schmid, PhD <schmid@uniroma3.it>
%   BioLab, Dept Applied Electronics, University Roma TRE, Italy
%   Motion Analysis Lab, Spaulding Rehab Hospital, Boston, MA, USA
%   
%   Last Updated: June 27, 2007
%   Please retain the author names, and give acknowledgement where necessary.
%   The author does not assume any responsibility related to the use
%   of this code, and does not guarantee its correctness.
%

%%   Phase 1: Preprocess data:
%   1. Remove mean
%   2. Low pass filter data (FIR filter, 300 samples, 10 Hz)

if nargin < 2
    disp('Not enough input data:')
    disp('at least two variables, i.e. CoPAP and CoPML, are needed.')
    return
end

if nargin == 2
    Fsamp = 100;
elseif isempty(Fsamp)
    Fsamp = 100;
end


LPfilt = fir1(300,10/(Fsamp/2));
score(:,1) = tm(CoPML);
score(:,2) = tm(CoPAP);
score(:,1) = filtfilt(LPfilt,1,score(:,1));
score(:,2) = filtfilt(LPfilt,1,score(:,2));

%% Phase 2: choose stability limits
if nargin < 4
    g = 2*range(score(:,1));
    h = 2*range(score(:,2));
elseif isempty(Bound)
    g = 2*range(score(:,1));
    h = 2*range(score(:,2));
else
    g = Bound(1);
    h = bound(2);
end

%% Phase 3. Extract TtB function, based on parabolic motion 


velCoPML = [0; diff(score(:,1))].*Fsamp;
velCoPAP = [0; diff(score(:,2))].*Fsamp;
accCoPML =[0; diff(filtfilt(LPfilt,1,velCoPML))].*Fsamp;
accCoPAP=[0; diff(filtfilt(LPfilt,1,velCoPAP))].*Fsamp;

TtB=zeros(size(score,1),1);

for ii=3:size(score,1)
    a=accCoPML(ii);
    b=velCoPML(ii);
    c=score(ii,1);
    d=accCoPAP(ii);
    e=velCoPAP(ii);
    f=score(ii,2);
    par(1)=(a^2)*(h^2)+(d^2)*(g^2);
    par(2)=2*a*b*(h^2)+2*d*e*(g^2);
    par(3)=(b^2)*(h^2)+2*a*c*(h^2)+(e^2)*(g^2)+2*d*f*(g^2);
    par(4)=2*b*c*(h^2)+2*e*f*(g^2);
    par(5)=(c^2)*(h^2)+(f^2*g^2)-(g^2*h^2);
    temp=roots(par);
    TtB(ii)=min(temp((real(temp)>0&imag(temp)==0)));
end
TtB(1:2,1)=TtB(3,1);


%% Accessory inner function to remove mean from data
function m=tm(matrix)
%function m=tm(matrix,rows)
%Subtracts column offset from a matrix;
%rows+1 does the same on the rows
if nargin==2
    matrix=matrix';
end
[N M]=size(matrix);
m=zeros(N,M);
mean_mat=mean(matrix);
for i=1:N
    m(i,:)=matrix(i,:)-mean_mat;
end
if nargin==2
    m=m';
end
